import sys
import numpy as np
import cv2

def seed_and_grow(img):

    num_superpixels = 800
    num_iterations = 10
    prior = 2
    num_levels = 4
    num_histogram_bins = 5

    converted_img = cv2.cvtColor(img, cv2.COLOR_BGR2HSV)
    height,width,channels = converted_img.shape

    seeds = cv2.ximgproc.createSuperpixelSEEDS(width, height, channels, num_superpixels, num_levels, prior, num_histogram_bins)

    seeds.iterate(converted_img, num_iterations)

    # retrieve the segmentation result
    labels = seeds.getLabels()

    mask = seeds.getLabelContourMask(False)

    # stitch foreground & background together
    mask_inv = cv2.bitwise_not(mask)
    result_bg = cv2.bitwise_and(img, img, mask=mask_inv)
    color_img = np.zeros((height,width,3), np.uint8)
    color_img[:] = (0, 0, 255)
    result_fg = cv2.bitwise_and(color_img, color_img, mask=mask)
    result = cv2.add(result_bg, result_fg)
    
    cv2.imshow('result', result)
    cv2.imshow('mask', mask)
    cv2.waitKey(1)
    
    return labels

def graph_segmentation(img):

    sigma = 0.5 # 0.8
    k = 300
    min_size = 100
    gs = cv2.ximgproc.segmentation.createGraphSegmentation(sigma,k,min_size)
    labels = gs.processImage(img)
    
    cv2.imshow('image',img)
    cv2.waitKey(1)
    
    return labels

if __name__ == '__main__':
    try:
        segmentation_type = int(sys.argv[1])
    except:
        segmentation_type = 1

    img = cv2.imread("../lena.jpg",cv2.IMREAD_COLOR)

    if segmentation_type == 1: # region seed & grow
        labels = seed_and_grow(img)
        
    elif segmentation_type == 2: # graph segmentation
        labels = graph_segmentation(img)

    print("please, wait ...")
    label_count = np.max(labels)+1
    colors = np.array([(0,0,0) for i in range(label_count)],np.int)
    counts = np.array([(0,) for i in range(label_count)],np.int)
    for y in range(labels.shape[0]):
        for x in range(labels.shape[1]):
            colors[labels[y,x]] += img[y,x]
            counts[labels[y,x]] += 1
    colors = colors / counts
    colors = np.asarray(colors,np.uint8)
    result_labels = np.zeros(img.shape,np.uint8)
    for y in range(labels.shape[0]):
        for x in range(labels.shape[1]):
            result_labels[y,x] = colors[labels[y,x]]
            
    cv2.imshow('labels', result_labels)
    print('press any key ...')
    cv2.waitKey(0)
    cv2.destroyAllWindows()
