import numpy as np
import cv2

def grabcut(img,rect):
    global mask, value, pressed
    mask = np.zeros(img.shape[:2], dtype = np.uint8)
    value = 0
    pressed = False
    
    def mouseHandler2(event, x, y, flags, param):
        global mask, value, pressed
        if event == cv2.EVENT_LBUTTONDOWN:
            pressed = True
        elif event == cv2.EVENT_LBUTTONUP:
            pressed = False
        if pressed:
            #mask[y,x] = value
            cv2.circle(mask,(x,y),3,value,cv2.FILLED)
            
    cv2.namedWindow("grabcut")
    cv2.setMouseCallback("grabcut", mouseHandler2)
    cv2.namedWindow("output")
    cv2.setMouseCallback("output", mouseHandler2)
    
    #print("press B for Background, b for probable background,")
    #print("F for Foregroud and f for probable foreground")
    #print("then draw and press n for the next iteration")

    iter = 0
    n = 1
    bgdmodel = np.zeros((1, 65), np.float64)
    fgdmodel = np.zeros((1, 65), np.float64)
    while True:
        if n > iter:
            if iter == 0: # grabcut with rect
                cv2.grabCut(img, mask, rect, bgdmodel, fgdmodel, 1, cv2.GC_INIT_WITH_RECT)
            else: # grabcut with mask
                cv2.grabCut(img, mask, rect, bgdmodel, fgdmodel, 1, cv2.GC_INIT_WITH_MASK)
            iter += 1
        
        final_mask = np.where((mask==1) + (mask==3), 255, 0).astype('uint8')
        output = cv2.bitwise_and(img, img, mask=final_mask)
        cv2.imshow("output",output)

        res = np.copy(img)
        res[mask == 0] = (0,255,0)
        res[mask == 1] = (0,0,255)
        res[mask == 2] = (255,255,0)
        res[mask == 3] = (255,0,255)
        cv2.imshow("grabcut",res)
        key = cv2.waitKey(10)
        
        if key == ord('B'):
            value = 0
            print('select Background')
        elif key == ord('F'):
            value = 1
            print('select Foreground')
        elif key == ord('b'):
            value = 2
            print('select probable background')
        elif key == ord('f'):
            value = 3
            print('select probable foreground')
        elif key == ord('n'):
            n += 1
            print('next iteration, you may press Esc to escape')
        elif key == 27:
            break
    
    cv2.rectangle(img,rect,(0,255,0),2)
    cv2.imwrite("grabcut-rect.png",img)
    cv2.imwrite("grabcut-mask.png",res)
    cv2.imwrite("grabcut-result.png",output)
    cv2.destroyWindow("output")
    cv2.destroyWindow("grabcut")
    return output
    
if __name__ == '__main__':

    # selection of the target 
    refPt0 = refPt1 = None
    selPt0 = selPt1 = None
    def mouseHandler(event, x, y, flags, param):
        global refPt0, refPt1, selPt0, selPt1
        if event == cv2.EVENT_LBUTTONDOWN:
            refPt0 = (x, y)
        elif event == cv2.EVENT_LBUTTONUP:
            selPt0 = refPt0
            selPt1 = (x,y) 
            refPt0 = refPt1 = None
        elif refPt0 is not None:
            refPt1 = (x,y)
            
    cv2.namedWindow("selection")
    cv2.setMouseCallback("selection", mouseHandler)

    frame = cv2.imread("../Face.jpg",cv2.IMREAD_COLOR)

    print('select rectangle with an object or press k to escape')
    rect = None
    while True:
        res = np.copy(frame)
        if selPt0 is not None and selPt1 is not None:
            tl = (min(selPt0[0],selPt1[0]),min(selPt0[1],selPt1[1]))
            br = (max(selPt0[0],selPt1[0]),max(selPt0[1],selPt1[1]))
            #print(tl,br)
            target = frame[tl[1]:br[1],tl[0]:br[0]]
            if len(target) > 0:
                rect = (tl[0],tl[1],br[0]-tl[0]+1,br[1]-tl[1]+1)
            selPt0 = setPt1 = None
        elif refPt0 is not None and refPt1 is not None:
            cv2.rectangle(res,(min(refPt0[0],refPt1[0]),min(refPt0[1],refPt1[1])),(max(refPt0[0],refPt1[0]),max(refPt0[1],refPt1[1])),(0,255,0),1)
            
        if rect is not None:
            #print(rect)
            grabcut(np.copy(frame),rect)
            print('select rectangle with an object or press k to escape')
            rect = None
            
        cv2.imshow("selection", res)
        key = cv2.waitKey(10)
        if key == ord('k'):
            break
     
    cv2.destroyAllWindows()
